//---------------------------------------------------------------------------

#pragma hdrstop

#include "Polygons.h"
#include "Triangle.h"
#include "Line.h"
#include "untDraw.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)


Polygons::Polygons():
  count(3),
  vertices(new Point2D[3]),
  center(Point2D())
{
   cout << "\nContrutor Polygons - Default";
}

Polygons::Polygons(int count, Point2D center, Point2D vertice):
  count(count),
  vertices(new Point2D[count]),
  center(center)
{
  cout << "\nContrutor Polygons - Regular Polygons";
  
  for (int i = 0; i < count; i++)
    (vertices[i] = vertice).RotateAround(i * 2 * 3.1415926535 / count, center);

}

Polygons::Polygons(int count, const Point2D points[]):
  count(count),
  vertices(new Point2D[count])
{
  for (int i = 0; i < count; i++)
    vertices[i] = points[i];

  double Xm=vertices[0].X();
  double Ym=vertices[0].Y();
  double XM=vertices[0].X();
  double YM=vertices[0].Y();
  for(int i=1;i<count;i++)
  {
     if (vertices[i].X()<Xm)
     Xm=vertices[i].X();
     if (vertices[i].Y()<Ym)
     Ym=vertices[i].Y();
     if (vertices[i].X()>XM)
     XM=vertices[i].X();
     if (vertices[i].Y()>YM)
     YM=vertices[i].Y();
  }
  center=Point2D((Xm+XM)/2,(Ym+YM)/2);
}

Polygons::Polygons(int count, const double x[], const double y[]):
  count(count),
  vertices(new Point2D[count])
{
  for (int i = 0; i < count; i++)
    vertices[i] = Point2D(x[i], y[i]);

  double Xm=vertices[0].X();
  double Ym=vertices[0].Y();
  double XM=vertices[0].X();
  double YM=vertices[0].Y();
  for(int i=1;i<count;i++)
  {
     if (vertices[i].X()<Xm)
     Xm=vertices[i].X();
     if (vertices[i].Y()<Ym)
     Ym=vertices[i].Y();
     if (vertices[i].X()>XM)
     XM=vertices[i].X();
     if (vertices[i].Y()>YM)
     YM=vertices[i].Y();
  }
  center=Point2D((Xm+XM)/2,(Ym+YM)/2);
}

Polygons::Polygons(const Polygons& other):
  count(other.count),
  vertices(new Point2D[other.count])
{
  cout << "\nContrutor Polygons - Copy ";
  for (int i = 0; i < other.count; i++)
    vertices[i] = other.vertices[i];
}
//---------------------------------------------------------------------------
Polygons::~Polygons()
{
   cout << "\nDestructor Polygons ";
  delete [] vertices;
}
//---------------------------------------------------------------------------
Point2D Polygons::Center() const
{
   return center;
}

void Polygons::SetCenter(const Point2D& newCenter)
{
   center = newCenter;
}

int Polygons::Count() const
{
  return count;
}

Point2D Polygons::Vertex(int x) const
{
  return vertices[x];
}

 Point2D& Polygons::operator [](int x) 
{
  return vertices[x];
}

void Polygons::Set(const Point2D& p, int x)
{
  vertices[x] = p;
}
//---------------------------------------------------------------------------
void Polygons::Scale(double fx, double fy)
{
  for (int i = 0; i < count; i++)
    vertices[i].Scale(fx, fy);
  center.Scale(fx, fy);
}

void Polygons::Translate(double dx, double dy)
{
  for (int i = 0; i < count; i++)
    vertices[i].Translate(dx, dy);
  center.Translate(dx,dy);
}

void Polygons::Rotate(double angle)
{
  for (int i = 0; i < count; i++)
    vertices[i].Rotate(angle);
  center.Rotate(angle);
}

Point2D Polygons::TopLeftLimit()
{
  double Xm=vertices[0].X();
  double Ym=vertices[0].Y();
  for(int i=1;i<count;i++)
  {
    if (vertices[i].X()<Xm)
      Xm=vertices[i].X();
    if (vertices[i].Y()<Ym)
      Ym=vertices[i].Y();
  }
  return Point2D(Xm,Ym);
}

Point2D Polygons::BottomRightLimit()
{
  double XM=vertices[0].X();
  double YM=vertices[0].Y();
  for(int i=1;i<count;i++)
  {
    if (vertices[i].X()>XM)
      XM=vertices[i].X();
    if (vertices[i].Y()>YM)
      YM=vertices[i].Y();
  }
  return Point2D(XM,YM);
}


//---------------------------------------------------------------------------
void Polygons::Write(ostream& output) const
{
  output << count;
  for (int i = 0; i < count; i++)
  {
    output << " ";
    vertices[i].Write(output);
  }
}

void Polygons::WriteLine(ostream& output) const
{
  Write(output);
  output << endl;
}
//-------------------------------------------------------------------------
void Polygons::Copy(const Polygons& other)
{
  if (this != &other)
  {
    delete [] vertices;
    vertices = new Point2D[other.count];
    count = other.count;
    for (int i = 0; i < other.count; i++)
      vertices[i] = other.vertices[i];
  }
}


Polygons& Polygons::operator = (const Polygons& other)
{
  Copy(other);
  return *this;
}
//-------------------------------------------------------------------------

bool Polygons::Convex() const
{
  for (int i = 1; i < count; i++)
  {
    if (vertices[i] == vertices[(i+1)%count])
      return false;
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(vertices[(i-1)%count], vertices[(i+2)%count]))
        return false;
   }
  return true;
}

bool Polygons::Contains (const Point2D& p)
{
  for (int i = 0; i < count; i++)
  {
    if (vertices[i] == vertices[(i+1)%count])
      return false;
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(Center(), p))
        return false;
   }
  return true;/*

  Point2D *v;
  v=new Point2D [20];
  for (int i = 0; i < count; i++)
  {
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(vertices[(i-1)%count], vertices[(i+2)%count]))
        v[i+1]=vertices[i+1];
   }


  double dis=9999999999.0;
  int pt=0;
  for (int i = 0; i < count; i++)
  {
     if (p.DistanceTo(vertices[i])<dis && v[i]!=vertices[i]){
       dis=p.DistanceTo(vertices[i]);
       pt=i;
     }
  }

  Point2D p1=vertices[pt];
  Point2D p2=vertices[(pt-1)%count];
  Point2D p3=vertices[(pt+1)%count];

  frmDraw->StatusBar->SimpleText="p1: "+FloatToStr(p1.X())+","+p1.Y()+
                                "    p2: "+p2.X()+","+p2.Y()+
                                "    p3: "+p3.X()+","+p3.Y();




  for (int i = 0; i < count; i++)
  {
    if (vertices[i] == vertices[(i+1)%count])
      return false;
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(p1, p))
        return false;
   }
   for (int i = 0; i < count; i++)
  {
    if (vertices[i] == vertices[(i+1)%count])
      return false;
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(p2, p))
        return false;
   }
   for (int i = 0; i < count; i++)
  {
    if (vertices[i] == vertices[(i+1)%count])
      return false;
    Line side(vertices[i], vertices[(i+1)%count]);
    if (!side.SameSide(p3, p))
        return false;
   }


  return true;  */

}
